.g-thumbnail-size-input
  border-radius 3px
  padding 5px 10px
  font-size 12px
  border 1px solid #ccc
  width 90px
  vertical-align middle

.g-thumbnail-size-between
  margin 0 9px
  vertical-align middle
  font-size 14px

.g-thumbnail-crop-container
  margin-left 20px

.g-thumbnail-size-form-container
  margin-bottom 15px

.g-thumbnail-custom-target-container
  margin-bottom 8px

  .g-target-result
    padding 5px 10px
    background-color #eee
    border-radius 3px
    margin-bottom 6px
